CON
   _clkmode = xtal1 + pll16x
   _xinfreq = 5_000_000
   
   bootRx   = 30
   bootTx   = 31
   
'' SRAM I/O pins

   SRAM_Dta = 4              ' SRAM Data In/Out
   SRAM_Clk = 5              ' SRAM Clock
   SRAM_CS0 = 6              ' SRAM Chip Select 0
   SRAM_CS1 = 7              ' SRAM Chip Select 1

'' SRAM command codes

   SRAM_Wr  = %0000_0010     ' SRAM Write Command
   SRAM_Rd  = %0000_0011     ' SRAM Read Command
   SRAM_Dis = %0000_0100     ' SRAM Write Disable
   SRAM_Ena = %0000_0110     ' SRAM Write Enable

OBJ
   ser : "FullDuplexSerial"  ' Serial I/O
   
VAR
   long cog, lock, data
   
PUB start : ok
   ser.start(bootRx,bootTx,0,9600)     ' Start debug serial I/O
   data := lock := locknew + 1         ' Reserve a semaphore (lock)
   cog := ok := cognew(@eEntryPoint,@data)
   repeat until data == 0              ' Wait for cog to initialize itself

PRI readData(addr)
   repeat until data.byte == 0         ' Wait for cog to be available
   data := addr << 16 | 1              ' Request a read
   repeat until data.byte == 0         ' Wait for read to finish
   return data >> 8

PRI writeData(addr,value)
   repeat until data.byte == 0         ' Wait for cog to be available
   data := addr << 16 | value << 8 | 2 ' Request a write
   repeat until data.byte == 0         ' Wait for write to finish
   
DAT
eEntryPoint
eSramLock      rdlong  eSramLock,PAR               ' Initially pass in the
eSramAddr      sub     eSramLock,#1                '  number of the lock + 1
eSramData      and     eSramLock,#7
eSramMask      mov     eSramMask,#0                ' Afterwards, the 1st byte
eSelect        mov     eSelect,#0                  '  is 0 when idle, 1 to read
eZero          mov     eZero,#0                    '   and 2 to write to SRAM
eCommand       wrlong  eZero,PAR

eActionWait    rdlong  eCommand,PAR                ' Wait for command request
               mov     eSramAddr,eCommand
               and     eCommand,#$FF
               cmp     eCommand,#1             wz  ' Read data
          if_e jmp     #eActionRead
               cmp     eCommand,#2             wz  ' Write data
         if_ne jmp     eActionWait

               mov     eSramData,eSramAddr         ' 2nd byte is data to write
               shr     eSramData,#8
               and     eSramData,#$FF
               shr     eSramAddr,#16               ' 2nd word is data address
               call    #eSramWrite                 ' Write the data to SRAM
               wrlong  eZero,PAR                   ' Mark write as done
               jmp     #eActionDone

eActionRead    shr     eSramAddr,#16               ' 2nd word is data address
               call    #eSramRead                  ' Read the data from SRAM
               shl     eSramData,#8                ' Pass the data back
               wrlong  eSramData,PAR               '  and mark as done
               jmp     #eActionWait

'' Read a byte from the SRAM.  SRAM address in eSramAddr.  Returns
'' value in eSramData.  Locks SRAM using eSramLock while accessing SRAM.

eSramRead      lockset eSramLock               wc  ' Wait for SRAM lock then
         if_nc jmp     #eSramRead                  '  ok to set up I/O pins
               mov     outa,#|<SRAM_CS0 + |<SRAM_CS1
               mov     dira,#|<SRAM_CS0 + |<SRAM_CS1 + |<SRAM_Clk
               mov     eSelect,#|<SRAM_CS0
               test    eSramAddr,eAddrMSBit    wz  ' Select proper SRAM chip
         if_nz mov     eSelect,#|<SRAM_CS1
               mov     eSramData,eReadCmd          ' First byte is read command
               or      eSramData,eSramAddr         ' Next two bytes are address
               mov     eSramMask,eReadMsk          ' Start with MSB of 1st byte
               or      dira,#|<SRAM_Dta            ' Make data pin an output
               andn    outa,eSelect                ' Enable chip select
eSramRead1     test    eSramMask,eSramData     wc  ' Copy data bit to I/O pin
               muxc    outa,#|<SRAM_Dta
               or      outa,#|<SRAM_Clk            ' Active on low->high edge
               andn    outa,#|<SRAM_Clk
               shr     eSramMask,#1            wz  ' Advance mask and
         if_nz jmp     #eSramRead1                 '  continue if more bits
               andn    dira,#|<SRAM_Dta            ' Make data pin an input
eSramRead2     or      outa,#|<SRAM_Clk
               test    eDataMsk,ina            wc  ' Copy data bit from I/O pin
               andn    outa,#|<SRAM_Clk            ' Active on high->low edge
               muxc    eSramData,eSramMask
               shr     eSramMask,#1            wz  ' Advance mask and
         if_nz jmp     #eSramRead2                 '  continue if more bits
               mov     dira,#0                     ' Release I/O pins & disable
               lockclr fSramLock                   ' Release SRAM lock
eSramReadHL_ret
eSramRead_ret  ret

'' Write a byte to the SRAM.  SRAM address in eSramAddr.  SRAM data in
'' eSramData.  Locks SRAM using eSramLock while accessing SRAM.

eSramWrite     lockset eSramLock               wc  ' Wait for SRAM lock then
         if_nc jmp     #eSramWrite                 '  ok to set up I/O pins
               mov     outa,#|<SRAM_CS0 + |<SRAM_CS1
               mov     dira,#|<SRAM_CS0 + |<SRAM_CS1 + |<SRAM_Clk
               mov     eSelect,#|<SRAM_CS0
               test    eSramAddr,eAddrMSBit    wz  ' Select proper SRAM chip
         if_nz mov     eSelect,#|<SRAM_CS1
               mov     eSramMask,#%10000000        ' 1st have to enable writes
               or      dira,#|<SRAM_Dta            ' Make data pin an output
               andn    outa,eSelect                ' Enable chip select
eSramEnable    test    eSramMask,#SRAM_Ena         ' Copy data bit to I/O pin
               muxc    outa,#|<SRAM_Dta
               or      outa,#|<SRAM_Clk            ' Active on low->high edge
               andn    outa,#|<SRAM_Clk
               shr     eSramMask,#1            wz  ' Advance mask and
         if_nz jmp     #eSramEnable                '  continue if more bits
               or      outa,eSelect                ' Disable chip select
               shl     eSramData,#24
               or      eSramData,eSramAddr         ' 1st byte is write command
               rol     eSramData,#8                ' Next two bytes are address
               or      eSramData,eWriteCmd         ' Last byte is data
               mov     eSramMask,eWriteMsk         ' Start with MSB of 1st byte
               or      dira,#|<SRAM_Dta            ' Make data pin an output
               andn    outa,eSelect                ' Enable chip select
eSramWrite4    test    eSramMask,eSramData     wc  ' Copy data bit to I/O pin
               muxc    outa,#|<SRAM_Dta
               or      outa,#|<SRAM_Clk            ' Active on low->high edge
               andn    outa,#|<SRAM_Clk
               shr     eSramMask,#1            wz  ' Advance mask and
         if_nz jmp     #eSramWrite4                '  continue if more bits
               mov     dira,#0                     ' Release I/O pins & disable
               lockclr fSramLock                   ' Release SRAM lock
eSramWriteHL_ret
eSramWrite_ret ret
